module rmo_tools
implicit none
contains
!__________________________
! RMO time at this location     
function t_rmo(s,z,a,eta,x,p,full) result(t)
real    :: s,z,a,eta,x,p,t
real    :: num1,num2,den1,den2,rmo
integer :: full
select case(full) ! RMO expression to be used
  case(1) !=== Full rmo 
    num1 = (s-1)*sin(x)**2                      
    !num2 = sqrt( 1 - sin(a)**2 * cos(eta-p)**2 )
    num2 = sqrt( 1 - sin(a)**2 * cos(eta-p)**2 )
    !den1 = (1-s*(1-num2))*cos(a)
    den1 = cos(a)
    den2 = num2**2-sin(x)**2                    
    !rmo  = num1*num2/den1/den2
    rmo  = num1/den1/den2
  case(2) !=== Dip reflection (eta=phi) = 2D RMO
    num1 = (s-1)*sin(x)**2
    !den1 = 1-s*(1-cos(a))
    den1 = cos(a)
    den2 = cos(a)**2 - sin(x)**2
    rmo  = num1/den1/den2
  case(3) !=== Strike reflection (eta-phi=90)
    num1 = (s-1)*tan(x)**2
    den1 = cos(a)
    rmo  = num1/den1
  case(4) !=== Flat reflector (alpha=0)
    rmo  = (s-1)*tan(x)**2
  case default 
    write(0,*)'FULL must be 1,2,3 or 4: full=',full
    call exit(-1)
end select
t = (-rmo*cos(a)+1)*z ! the cos is for a movement
            ! along the normal, not the vertical
end function 
!_______________________________
! Nearest neighbor interpolation
function near_neig(d,t,nt,dt,ot) result(m)
integer :: it,nt
real    :: t,dt,ot,m
real,allocatable,dimension(:)   :: d
it = 1.5+(t-ot)/dt
if((it<=nt).and.(it>0))then
  m = d(it)
end if 
end function 
!_____________________
! Linear interpolation
function lin_int(d,t,nt,dt,ot) result(m)
integer :: it_inf,nt
real    :: t,dt,ot,it_exa,frac,m
real,allocatable,dimension(:)   :: d
it_exa = 1.5+(t-ot)/dt
it_inf = it_exa 
frac   = it_exa-it_inf
if(((it_inf+1)<=nt).and.(it_inf>0))then
  m = (1-frac) * d(it_inf) + frac * d(it_inf+1)
end if 
end function 
!===============================
end module rmo_tools
